/*
  gDisplay_volt.c
    October 10, 2015
  * A Simple Menu that uses a COG.
  * Displays a Simple Menu and is now running
  * the voltage gauge in the status line.
*/
#include "simpletools.h"
#include "badgetools.h"
#include "datetime.h"

/* Voltage gauge*/
float calcVt(float batteryVolts, int decayMicros);
float batVolts(float vt);

float RC = 360000.0 * (1.0 / 100000000.0);
float vt, vbat;
/****************/

/* Case statement variables */
int states;
volatile int myLock;

/* Function declarations */
void NewClear();  // Clears screen lines 0 - 6.
void bLine();  // Clean a line.

/* COG */
void System_base();
/******************/

int main()
{
  // Add startup code here.
  badge_setup();
  
  myLock=locknew();
  lockclr(myLock);
/* Voltage RCT value. */
  vt = calcVt(4.17, 1453);
/******************/

  clear();
/* Setup text default font size.*/
// To be used in the COG, at startup.  
  text_size(SMALL);


/* Start the system COG. */  
  cog_run(System_base,128);
 
  while(1)
  {
    // Add main loop code here.
    /* Using continue in the switch statement
       brings you back to this point. */
    while(lockset(myLock));  // Lockit
    NewClear();  // Clear 0 - 6 and set cursor to (0,0)
    text_size(SMALL);
    cursor(0,0);
    oledprint("Any BTN Menu    ");
    lockclr(myLock);  // Release it.
         
    while(!buttons());  // No button press.
    /* Menu setup. */
    while(lockset(myLock));  // Lockit
    NewClear();
    oledprint("    MENU      ");
    cursor(0,1);
    oledprint("P17-button-P27");
    cursor(0,2);
    oledprint("P16-button-P26");
    cursor(0,3);
    oledprint("P15-button-P25");
    cursor(0,4);
    oledprint("      OSH     ");
    lockclr(myLock);  // Release it.
        
    while((states = buttons()) == 0);  // Wait for a button press.
    switch(states)
    {
      case 0b0100000:  //p17
        while(lockset(myLock));  // Lockit
        cursor(0,7);   // Use status line.
        oledprint("B17 ");
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0000001:  //P27
        while(lockset(myLock));  // Lockit
        cursor(0,7);   // Use status line.
        oledprint("B27 ");
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0010000:  //P16
        while(lockset(myLock));  // Lockit
        cursor(0,7);  // Use status line.
        oledprint("B16 ");
        lockclr(myLock);  // Release it.     
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0000010:  //P26
        while(lockset(myLock));  // Lockit
        cursor(0,7);   // Use status line.
        oledprint("B26 ");
        lockclr(myLock);  // Release it.     
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0001000:  //P15
        while(lockset(myLock));  // Lockit
        cursor(0,7);   // Use status line.
        oledprint("B15 ");
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0000100:  //P25
        while(lockset(myLock));  // Lockit
        cursor(0,7);   // Use status line.
        oledprint("B25 ");
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b1000000:  //OSH
        while(lockset(myLock));  // Lockit
        cursor(0,7);   // Use status line.
        oledprint("bOSH");
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)        
      /*********************/
      default:
        continue;    // Goes to the while(1)
    }      
    
  }  
}


/* COG for running system base utilty. */
void System_base()
{
  while(1)
  {
    /* Lockit */
    while(lockset(myLock));
    cursor(9,7);
    // Voltage
    vbat = batVolts(vt);
    pause(100);
    oledprint("%1.2f V", vbat);
    lockclr(myLock);
    /* Release it. */
  }      
}


float calcVt(float voltsBattery, int microsDecay)
{
  float voltsThreshold, tDecay;
  tDecay = ((float) microsDecay) / 1000000.0;
  voltsThreshold = voltsBattery * (1 - exp(-tDecay / RC));
  return voltsThreshold;
}
  
float batVolts(float voltsThreshold)
{
  float voltsBattery, microsDecay;
  low(0);
  pause(1);
  microsDecay = ((float) rc_time(0, 0)) / 1000000.0;
  voltsBattery = voltsThreshold / (1 - exp(-microsDecay / RC));
  return voltsBattery;
}

/* To be used for clearing lines 0 - 6 */
void NewClear()
{
  cursor(0,0);
  bLine();
  cursor(0,1);
  bLine();
  cursor(0,2);
  bLine();
  cursor(0,3);
  bLine();
  cursor(0,4);
  bLine();
  cursor(0,5);
  bLine();
  cursor(0,6);
  bLine();
  cursor(0,0);
}


/* Blanks a line. */
void bLine()
{
  oledprint("                ");
}    
